#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "stm8s.h"
#include "usrlib-uart.h"

void UART_Menu(void);

// address of data on EEPROM  (offset calinration values)
char* EEPROM_OFFSET_AIN0 = (char*)0x4000;
char* EEPROM_OFFSET_AIN1 = (char*)0x4010;
char* EEPROM_OFFSET_AIN2 = (char*)0x4020;
char* EEPROM_OFFSET_AIN3 = (char*)0x4030;
char* EEPROM_OFFSET_AIN4 = (char*)0x4040;
char* EEPROM_OFFSET_AIN5 = (char*)0x4050;
char* EEPROM_OFFSET_AIN6 = (char*)0x4060;
char* EEPROM_OFFSET_AIN7 = (char*)0x4070;
char* EEPROM_OFFSET_AIN8 = (char*)0x4080;
char* EEPROM_OFFSET_AIN9 = (char*)0x4090;

// work area
char UARTstrTX[80];
char dump_ascii[4];
char dump_char[17];
float offset_AIN[10];
int i,k;

void main(void)
{
	CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSE, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE);
	
	TIM3_DeInit();
	TIM3_TimeBaseInit(TIM3_PRESCALER_512,3124); // 0.1sec timer
	TIM3_ITConfig(TIM3_IT_UPDATE, ENABLE);
	TIM3_Cmd(ENABLE);

  ADC1_DeInit();
  ADC1_Init(ADC1_CONVERSIONMODE_CONTINUOUS, ADC1_CHANNEL_9, ADC1_PRESSEL_FCPU_D4, ADC1_EXTTRIG_TIM, DISABLE, ADC1_ALIGN_RIGHT, ADC1_SCHMITTTRIG_ALL, DISABLE);
	ADC1_ScanModeCmd(ENABLE);
	ADC1_DataBufferCmd(ENABLE);
	ADC1_StartConversion();

	UART2_DeInit();
  UART2_Init((u32)115200, UART2_WORDLENGTH_9D, UART2_STOPBITS_1, UART2_PARITY_ODD, UART2_SYNCMODE_CLOCK_DISABLE, UART2_MODE_TXRX_ENABLE);
  UART2_ITConfig(UART2_IT_RXNE_OR, ENABLE);
	UART2_Cmd(ENABLE);

	GPIO_Init(GPIOD, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_FAST); // LED

	offset_AIN[0]=atof(EEPROM_OFFSET_AIN0);
	offset_AIN[1]=atof(EEPROM_OFFSET_AIN1);
	offset_AIN[2]=atof(EEPROM_OFFSET_AIN2);
	offset_AIN[3]=atof(EEPROM_OFFSET_AIN3);
	offset_AIN[4]=atof(EEPROM_OFFSET_AIN4);
	offset_AIN[5]=atof(EEPROM_OFFSET_AIN5);
	offset_AIN[6]=atof(EEPROM_OFFSET_AIN6);
	offset_AIN[7]=atof(EEPROM_OFFSET_AIN7);
	offset_AIN[8]=atof(EEPROM_OFFSET_AIN8);
	offset_AIN[9]=atof(EEPROM_OFFSET_AIN9);

  enableInterrupts();
	SerialPutString("=====start temperature log=====\r\n");

	while (1){ UART_Menu();	} // Main loop
}

u16 adc[10]; // ADC0-9 values
u32 adc_sum[10];  // ADC0-9 sum values
float mV[10]; // ADC0-9 mV values
float temperature[10]; // calculated temperature
char VERBOSE='s';   // silent or verbose
float INTERVAL=1;     // display interval second
u16 cnt=0; // 0-9 to counte 1sec
u16 adc_sum_cnt; // AINx summation times
u32 count_01sec;    // interrupt counter (0.1sec)
u8 now_hour, now_min, now_sec;

void Tim3Update(void) interrupt 15 // 0.1Sec interrupt
{
	GPIO_WriteReverse(GPIOD,GPIO_PIN_0); // LED flash
	count_01sec++;
	cnt++;
	if(cnt>(10*INTERVAL)){
		cnt=0; // 1sec counter

		// calculations of voltage and temperature
		for(i=0;i<=9;i++)
		{
			mV[i] = (float)adc_sum[i]/(float)adc_sum_cnt/1024.0*3333;
			temperature[i] = ( mV[i] - 850 ) / 10 + 25 + offset_AIN[i];
		}

		// current time to display
		now_hour= (u8)((count_01sec/36000) % 24);
		now_min = (u8)((count_01sec/600) % 60);
		now_sec = (u8)((count_01sec/10) % 60);
		sprintf(UARTstrTX,"%02d:%02d:%02d ",(int)now_hour,(int)now_min,(int)now_sec);
		SerialPutString(UARTstrTX);
		for(i=0;i<=9;i++)
		{
			sprintf(UARTstrTX,"%d:%2.1f ",i,temperature[i]);
			SerialPutString(UARTstrTX);
		}
		SerialPutString("\r\n");
		if(VERBOSE=='v')
		{
			SerialPutString("ADC(0-9)=");
			for(i=0;i<=9;i++)
			{
				sprintf(UARTstrTX,"%d:%d ",i,(int)((float)adc_sum[i]/(float)adc_sum_cnt));
				SerialPutString(UARTstrTX);
			}
			SerialPutString("\r\n");
			SerialPutString("mV(0-9)=");
			for(i=0;i<=9;i++)
			{
				sprintf(UARTstrTX,"%d:%d ",i,(int)mV[i]);
				SerialPutString(UARTstrTX);
			}
			SerialPutString("\r\n");
		}
		// prepair for next turn
		adc_sum_cnt=0;
		for(i=0;i<=9;i++){ adc_sum[i]=0; }
	}
	else {
		adc_sum_cnt++;
		for(i=0;i<=9;i++)
		{
			adc[i]=ADC1_GetBufferValue(i);
			adc_sum[i]+=adc[i];
		}
		if(VERBOSE=='v'){
			SerialPutString("ADC(0-9)=");
			for(i=0;i<=9;i++){
				sprintf(UARTstrTX,"%d:%4d ",i,adc[i]);
				SerialPutString(UARTstrTX);
			}
			SerialPutString("\r\n");
		}
	}
	TIM3_ClearFlag(TIM3_FLAG_UPDATE);
}

void UART_Menu(void)
{
	int fieldnumber;
	if(UART_STR_EXIST==1) // there is a command line
	{
	  fieldnumber = separate_line(command);
		// verbose mode or not
		if(strcmp(Field[0],"v")==0 && fieldnumber==1)
		{
			(VERBOSE=='v') ? (VERBOSE='s') : (VERBOSE='v');
		}
		// set interval second
		else if(strcmp(Field[0],"int")==0 && fieldnumber==2)
		{
			if(atof(Field[1])<0.1)goto NG;
			else if(atof(Field[1])>60)goto NG;
			else INTERVAL = atof(Field[1]);
		}
		// set date and time
		else if(strcmp(Field[0],"time")==0 && fieldnumber==3)
		{
			u8 hour, min;
		  hour = (int)atof(Field[1]);
			min  = (int)atof(Field[2]);
			count_01sec = (u32)hour*60*60*10 + (u32)min*60*10;
		}
		// set offset values
		else if(strcmp(Field[0],"ofs")==0 && fieldnumber==3)
		{
			int AINx;
			AINx = (int)atof(Field[1]);
			if(AINx<0 || 9<AINx) goto NG; // check AINx
			else
			{
				FLASH_Unlock(FLASH_MEMTYPE_DATA);
				for(i=0;i<=15;i++)
				{
					FLASH_EraseByte(0x4000+AINx*16+i);
					FLASH_ProgramByte(0x4000+AINx*16+i,(u8)Field[2][i]);
				}
				FLASH_Lock(FLASH_MEMTYPE_DATA);
				offset_AIN[AINx]=atof(Field[2]);
			}
		}
		// dump EEPROM
		else if(strcmp(Field[0],"dump")==0 && fieldnumber==1)
		{
			disableInterrupts();
			SerialPutString("\r\n");
			for(i=0;i<=9;i++)
			{
				UARTstrTX[0]=0; // null string
				dump_char[16]=0;
				for(k=0;k<=15;k++)
				{
					uint8_t a;
					a=FLASH_ReadByte(0x4000+16*i+k);
					sprintf(dump_ascii,"%02x ",a);
					strcat(UARTstrTX,dump_ascii);
					if(a<32)a=46;
					if(a==0x7F)a=46;
					if((a&0xF0)==0x80)a=46;
					if((a&0xF0)==0x90)a=46;
					if((a&0xF0)==0xE0)a=46;
					if((a&0xF0)==0xF0)a=46;
					dump_char[k]=a;
				}
				SerialPutString(UARTstrTX);
				SerialPutString(" | ");
				SerialPutString(dump_char);
				SerialPutString("\r\n");
			}
		  enableInterrupts();
		}
		// clear EEPROM
		else if(strcmp(Field[0],"clr")==0 && fieldnumber==1)
		{
			disableInterrupts();
			SerialPutString("\r\n");
			FLASH_Unlock(FLASH_MEMTYPE_DATA);
			for(i=0;i<=0x9f;i++) FLASH_EraseByte(0x4000+i);
			FLASH_Lock(FLASH_MEMTYPE_DATA);
			for(i=0;i<=9;i++)offset_AIN[i]=0;
		  enableInterrupts();
		}
		else
		{
			NG:
			SerialPutString("\r\nCOMMAND NG !!!\r\n\n");
			SerialPutString("time 21 22 --> set time hour/min\r\n");
			SerialPutString("int 3      --> interval time 3 seconds\r\n");
			SerialPutString("ofs 4 -0.5 --> set AIN4 offset to -0.5\r\n");
			SerialPutString("v          --> switch verbose <--> silent\r\n");
			SerialPutString("dump       --> dump EEPROM\r\n");
			SerialPutString("clr        --> clear EEPROM\r\n");
		}
		UART_STR_EXIST=0;
	}
}

#ifdef USE_FULL_ASSERT
void assert_failed(uint8_t* file, uint32_t line)
{ 
  while (1)  {  }
}
#endif
